
from datetime import datetime
import logging

from tqdmpromproxy.snapshot import TqdmSnapshot


class PrometheusBucket:
    '''
    Collection of all grouped metrics'''

    def __init__(self, bucket: str, item_scale: str, retired_attrs: list[str], current_attrs: list[str]):
        self.bucket = bucket
        self.item_scale = item_scale
        self.current_attrs = list.copy(current_attrs)
        self.retired_attrs = list.copy(retired_attrs)

        self.aggregated = dict([(attr, 0) for attr in self.retired_attrs])

        self.known_instances = {}
        self.retired_instances = 0

    def matches(self, instance: TqdmSnapshot):
        return instance.bucket() == self.bucket

    def update(self, instance: TqdmSnapshot):
        if instance.identity() not in [i.identity() for i in self.known_instances]:
            self.known_instances[instance] = 0

        self.known_instances[instance] = datetime.now()

    # pylint: disable=unnecessary-dunder-call
    def to_prometheus_lines(self):
        '''Return the metrics as a prometheus string'''
        yield f"# TQDM group {self.bucket} "
        yield f"{self.bucket}_active_count {len(self.known_instances)}"
        yield f"{self.bucket}_finished_count {self.retired_instances}"

        yield f"## Individual properties with scale {self.item_scale}"
        for prop in self.current_attrs:

            val = sum([instance.__getattribute__(prop) or 0
                      for instance in self.known_instances.keys()])

            if prop in self.retired_attrs:
                val += self.aggregated[prop]

            yield f"{self.bucket}_{prop}_{self.item_scale} {val}"

    # pylint: disable=unnecessary-dunder-call
    def retire(self, instance: TqdmSnapshot):
        '''Move an instance from the active to the retired list'''
        if instance.identity() in [i.identity() for i in self.known_instances]:
            for prop in self.retired_attrs:
                self.aggregated[prop] += instance.__getattribute__(prop)

            del self.known_instances[instance]

            self.retired_instances += 1

    def prune(self, max_age: int):
        '''Remove any instances that are no longer active'''
        to_remove = []
        for key, value in self.known_instances.items():
            if (datetime.now() - value).total_seconds() > max_age:
                to_remove.append(key)

        for key in to_remove:
            self.retire(key)

    @classmethod
    def from_instance(cls, snapshot: TqdmSnapshot):
        return cls(snapshot.bucket(), snapshot.unit, ['completed'], ['completed', 'rate'])



from threading import Thread
from bottle import WSGIRefServer, run, route
from bottle import route, run, response, Bottle
from queue import Empty, Queue
from concurrent.futures import Future, ThreadPoolExecutor
import requests

import logging


class AsyncMetricServer:
    def __init__(self, queue: Queue, address: str="localhost", port: int=9000):
        self.address = address
        self.port = port

        self.snapshot: str = "# HELP No metric data has been collected yet. Check back soon\n"

        global TQDM_PROM_DATA
        TQDM_PROM_DATA = self.snapshot

        self.bottle = Bottle()
        self.bottle.route('/metrics')(self.metrics)

        self.queue = queue
        self.poller = ThreadPoolExecutor(max_workers=1)
        self.wsgi = None

        self.wsgi = WSGIRefServer(
            app=self.bottle, host=self.address, port=self.port)
        self.server = Thread(
            target=self.wsgi.run, args=(self.bottle, ), daemon=True)

        self._stop = False
        self.additional_content: str = ''

    def start(self):
        self.poller.submit(self.poll)
        self.server.start()

    def stop(self):
        self._stop = True
        
        if not hasattr(self.wsgi, 'srv'):
            # dirty force lazy load
            try:
                requests.get(f'http://{self.address}:{self.port}/shutdown', timeout=0.1)
            except requests.exceptions.ConnectTimeout:
                pass
            finally:
                pass

        if hasattr(self.wsgi, 'srv'):
            self.wsgi.srv.shutdown()
        self.server.join()

    def poll(self):
        last_found = 0
        while not self._stop:
            while not self.queue.empty():
                try:

                    item = self.queue.get(0.1)
                    self.snapshot = item
                    last_found = 0
                    logging.info("Polled queue")
                except Empty:
                    last_found += 1

                    if last_found % 10 == 0:
                        logging.info("No new items found in queue")

                finally:
                    pass

    @route('/metrics')
    def metrics(self):
        response.content_type = 'application/text; charset=utf-8'
        logging.info("Metrics requested, returning %d bytes" %
                     len(self.snapshot))
        return self.snapshot



import datetime
from tqdm import tqdm as native_tqdm


class TqdmSnapshot():
    '''Snapshot of the state of a tqdm instance'''

    def __init__(self,
                 item: native_tqdm,
                 bar_id: int = None,
                 time_ms: datetime.datetime = None,
                 desc: str = None,

                 n: int = None,
                 elapsed: int = None,
                 unit: str = None,
                 prefix: str = None,
                 unit_scale: str = None,
                 rate: float = None,
                 postfix: str = None,
                 unit_divisor: int = None,
                 initial: int = None,
                 total: int = None,
                 **_kwargs
                 ):
        # from our wrapper
        self.bar_id = bar_id
        self.item = item
        self.time_ms = time_ms

        # from tqdm api args
        self.completed = n
        self.total = total
        self.elapsed = elapsed
        self.prefix = prefix
        self.postfix = postfix
        self.unit = unit
        self.unit_scale = unit_scale
        self.unit_divisor = unit_divisor
        self.rate = rate
        self.initial = initial

        # from other/meta attributes
        self.desc = desc

    def __repr__(self):
        friendly = "Instance %s Bar = %s, Finished %s Total %s, throughput %s" % \
            (self.item, self.desc, self.completed, self.total, self.rate)

        # + " [ " + ', '.join(f"{prop}= {self.__getattribute__(prop)}" for prop in dir(self) if not prop.startswith("_")) + "]"
        return friendly

    def identity(self):
        '''Return a unique identifier for this instance.
        These properties should not change during the lifetime of the instance'''
        return str(self.bar_id) + self.prefix

    def bucket(self):
        '''Return a summary of the instance, so like instances can be aggregated
        For example all Gzip tasks, or all FileUpload tasks'''
        return self.prefix[0:self.prefix.find(' ') or 10]

    @classmethod
    def from_bar(cls, bar: native_tqdm):
        if not isinstance(bar, native_tqdm):
            raise ValueError("Expected a tqdm instance")

        return cls(bar,
                   bar_id=getattr(bar, 'pos', 0),
                   desc=getattr(bar, 'desc', ''),
                   time_ms=datetime.datetime.now(),
                   **bar.format_dict)



import datetime
import time
import unittest

from tqdm import tqdm as native_tqdm

from tqdmpromproxy.snapshot import TqdmSnapshot


class TestSnapshot(unittest.TestCase):

    def delaybody(self, seconds=0.1):
        time.sleep(seconds)

    def generate_bytes(self, kbytes):
        for _ in range(kbytes):
            yield b" " * 1024

    def test_manual(self):
        snapshot_time = datetime.datetime.now()
        snapshot = TqdmSnapshot(
            item=None,
            total=100,
            desc="Test Bar",
            n=50,
            time_ms=snapshot_time,
            unit="foo",
            bar_id=1
        )

        self.assertIsNone(snapshot.item)
        self.assertEqual(snapshot.total, 100)
        # self.assertEqual(snapshot.desc, "Test Bar")
        self.assertEqual(snapshot.completed, 50)
        self.assertEqual(snapshot.time_ms, snapshot_time)
        self.assertEqual(snapshot.bar_id, 1)
        self.assertEqual(snapshot.unit, "foo")
        self.assertIsNotNone(repr(snapshot))

    def test_from_bar_range(self):
        bar = native_tqdm(range(101), desc="Test Bar")
        bar.update(23)

        snapshot = TqdmSnapshot.from_bar(bar)

        self.assertEqual(snapshot.item, bar)
        self.assertEqual(snapshot.total, 101)
        self.assertEqual(snapshot.desc, "Test Bar")
        self.assertEqual(snapshot.completed, 23)
        self.assertIsNotNone(snapshot.time_ms)
        self.assertEqual(snapshot.bar_id, 0)

    def test_from_noiterator(self):
        bar = native_tqdm(desc="Checking prerequsites",
                          bar_format='{desc}', position=12, leave=False, unit="seconds waited")
        bar.update(23)

        snapshot = TqdmSnapshot.from_bar(bar)

        self.assertEqual(snapshot.item, bar)
        self.assertIsNone(snapshot.total)
        self.assertEqual(snapshot.desc, "Checking prerequsites")
        self.assertEqual(snapshot.completed, 23)
        self.assertIsNotNone(snapshot.time_ms)
        self.assertEqual(snapshot.bar_id, -12)

    def test_from_generator(self):
        keys = ["first", "second", "third", "fourth", "fifth"]
        values = [1, 2, 3, 4, 5]

        tuples = zip(keys, values)

        snaps = []
        for f, sz in (pbar := native_tqdm(tuples, desc="Task", unit='files', position=1, leave=False)):
            snaps.append((TqdmSnapshot.from_bar(pbar), f, sz))
            self.delaybody()

        counter = 0
        for snap, f, sz in snaps:
            self.assertEqual(snap.total, None)
            self.assertEqual(snap.desc, "Task")
            self.assertEqual(snap.completed, counter, f"Iteration {
                             counter} got {snap.completed}")
            self.assertEqual(snap.bar_id, -1)
            self.assertEqual(snap.unit, "files")

            print(f"Snap {snap} {f} {sz}")

            counter += 1

    def test_bytes(self):
        snaps = []
        target = 5 * 1024**3  # 1GB

        step_values = [0, 1, 2, 3, 4]
        units = ['B', 'B', 'KB', 'MB', 'GB']

        limit = len(step_values)-1

        steps = 3  # multiplies major unit to b make a number 'enough'

        with native_tqdm(self.generate_bytes(1), desc="byte(scale) through",
                         total=target,
                         unit='B', unit_scale=True) as pbar:

            for offset in range(limit):
                pbar.update(steps * 1024**step_values[offset])
                self.delaybody()
                snaps.append(TqdmSnapshot.from_bar(pbar))

                print(pbar)

        counter = 0
        running_total = 0
        for snap in snaps:
            self.assertEqual(snap.total, target)
            running_total += steps * 1024**counter
            self.assertEqual(snap.completed, running_total, f"Iteration {
                             counter} got {snap.completed}")
            # self.assertEqual(snap.unit, units[counter], f"Iteration {
            #                  counter} wanted {units[counter]} got {snap.unit}. possible (total so far {running_total})")

            print(f"Snap {snap} ")

            counter += 1





# capture tqdm updates via a process queue
# capture additional argments (network queue length, job queue length)
# expose a prometheus endpoint
# will possibly need to age tqdm as we wont get notified when they are 'done'

from concurrent.futures import ThreadPoolExecutor
from io import StringIO
import logging
from multiprocessing import Queue
from time import sleep

from tqdm import tqdm as native_tqdm

from tqdmpromproxy.bucket import PrometheusBucket
from tqdmpromproxy.metric_server import AsyncMetricServer
from tqdmpromproxy.snapshot import TqdmSnapshot


class TqdmPrometheusProxy():
    def __init__(self, http_host='localhost', http_port=9000, dump_files=0):
        '''
        Start a proxy
        dump_files = 0: no dump, 1= all, 2= all + iteration count 
        '''
        self.tqdm_events = Queue()
        self.tqdm_state = {}  # kv of tqdm_id:state
        self.tqdm_last_update = {}  # kv of tqdm_id:time
        self.http_port = http_port
        self.http_host = http_host
        self.raw_tqdm: list = []  # list of tqdm instances

        self.queue_handler = ThreadPoolExecutor(max_workers=1)
        self.http_handler = ThreadPoolExecutor(max_workers=1)
        self.http_server = AsyncMetricServer(self.tqdm_events, self.http_host, self.http_port)

        self.buckets: list[PrometheusBucket] = []
        self.dump_files = dump_files

    def tqdm(self, *args, **kwargs):
        instance = native_tqdm(*args, **kwargs)

        self.raw_tqdm += instance

        return instance

    def add(self, tqdm):
        self.raw_tqdm.append(tqdm)

    def remove(self, tqdm):
        self.raw_tqdm.remove(tqdm)

    def start(self):
        self.queue_handler.submit(self._poll)
        self.http_server.start()
        self.http_handler.submit(self.http_server)

    def _start_http_server(self):
        self.http_server.start()

    def __getattr__(self, name):
        return getattr(self.tqdm, name)

    def _poll(self):
        cycle = 0
        while not self.queue_handler._shutdown:
            logging.info("Polling %d instances" % len(self.raw_tqdm))

            try:
                now = self._collect()
                for item in now:
                    matched = False

                    for b in self.buckets:
                        if b.matches(item):
                            b.update(item)
                            matched = True
                            break

                    if not matched:
                        self.buckets.append(
                            PrometheusBucket.from_instance(item))

                logging.info("Polling collected")

            except KeyboardInterrupt:
                logging.info("Polling interrupted")
                break

            except Exception as e:
                logging.error("Error polling instances: %s" %
                              e, exc_info=True)

            finally:
                logging.info("Polling complete")
                sleep(0.3)

            buf = StringIO()
            self._dump_to_stream(buf)
            self.tqdm_events.put(buf.getvalue())

            if self.dump_files > 1:
                with open(f"data/metrics_all.txt", "wt", encoding="utf-8") as f:
                    self._dump_to_stream(f)

                if self.dump_files > 2:
                    with open(f"data/metrics.{str(cycle)}.txt", "wt", encoding="utf-8") as f:
                        self._dump_to_stream(f)
                    

            cycle += 1
            if cycle % 1000 == 0:
                for b in self.buckets:
                    b.prune(1)

    def _dump_to_stream(self, f):
    
        for b in self.buckets:

            for line in b.to_prometheus_lines():
                f.write(line)
                f.write("\n")

    def _collect(self) -> list[TqdmSnapshot]:
        '''Collect the current state of all known tqdm instances'''
        bars = []

        self._discover()

        for known_instance in self.raw_tqdm:
            for i in known_instance._instances:  # pylint: disable=protected-access
                snapshot = TqdmSnapshot.from_bar(i)
                logging.info("Snapshotted bar %s" % snapshot)
                bars.append(snapshot)

        return bars

    def _discover(self):
        '''Discover a new tqdm instance and add it to the known instances'''

        for t in self.raw_tqdm:
            for instance in t._instances:  # pylint: disable=protected-access
                if instance not in self.raw_tqdm:
                    self.add(instance)

    def stop(self):
        self.queue_handler.shutdown()
        self.http_server.stop()



from datetime import datetime
import unittest
from unittest import mock

from tqdmpromproxy.bucket import PrometheusBucket


class TestPrometheusBucket(unittest.TestCase):

    def setUp(self):
        bucket = "test_bucket"
        item_scale = "items"
        retired_attrs = ["completed"]
        current_attrs = ["completed", "rate"]

        self.vanilla_bucket = PrometheusBucket(
            bucket, item_scale, retired_attrs, current_attrs)

    def _create_mock_tqdm(self, name: str):
        instance = mock.Mock()
        instance.identity.return_value = f"new_instance_{name}"

        for attr in self.vanilla_bucket.current_attrs:
            setattr(instance, attr, 2)

        for attr in self.vanilla_bucket.retired_attrs:
            setattr(instance, attr, 1234)

        return instance

    def test_init(self):
        bucket = "test_bucket"
        item_scale = "items"
        retired_attrs = ["completed"]
        current_attrs = ["completed", "rate"]

        prom_bucket = PrometheusBucket(
            bucket, item_scale, retired_attrs, current_attrs)

        self.assertEqual(prom_bucket.bucket, bucket)
        self.assertEqual(prom_bucket.item_scale, item_scale)
        self.assertEqual(prom_bucket.current_attrs, current_attrs)
        self.assertEqual(prom_bucket.retired_attrs, retired_attrs)
        self.assertEqual(prom_bucket.aggregated, {"completed": 0})
        self.assertEqual(prom_bucket.known_instances, {})
        self.assertEqual(prom_bucket.retired_instances, 0)

    def test_matches(self):
        bucket = "test_bucket"
        item_scale = "items"
        retired_attrs = ["completed"]
        current_attrs = ["completed", "rate"]

        prom_bucket = PrometheusBucket(
            bucket, item_scale, retired_attrs, current_attrs)

        matching_instance = mock.Mock()
        matching_instance.bucket.return_value = bucket

        self.assertTrue(prom_bucket.matches(matching_instance))

        different_instance = mock.Mock()
        different_instance.bucket.return_value = "nope"

        self.assertFalse(prom_bucket.matches(different_instance))

    def test_update_single(self):

        new_instance = mock.Mock()
        new_instance.identity.return_value = "new_instance"

        self.assertEqual(len(self.vanilla_bucket.known_instances), 0)

        self.vanilla_bucket.update(new_instance)
        self.vanilla_bucket.update(new_instance)  # do it again

        self.assertTrue(new_instance
                        in self.vanilla_bucket.known_instances)
        self.assertEqual(len(self.vanilla_bucket.known_instances), 1)

    def test_update_multiple(self):

        instances = [self._create_mock_tqdm(i) for i in range(10)]

        self.assertEqual(len(self.vanilla_bucket.known_instances), 0)

        for i in instances:
            self.vanilla_bucket.update(i)

        self.assertEqual(
            len(self.vanilla_bucket.known_instances), len(instances))

    def test_retire(self):

        keep = self._create_mock_tqdm("keep")
        retire = self._create_mock_tqdm("retire")

        instances = [keep, retire]

        for i in instances:
            self.vanilla_bucket.update(i)

        self.vanilla_bucket.retire(retire)

        self.assertEqual(
            len(self.vanilla_bucket.known_instances), 1)
        self.assertEqual(self.vanilla_bucket.retired_instances, 1)

        for attr in self.vanilla_bucket.retired_attrs:
            self.assertEqual(self.vanilla_bucket.aggregated[attr], 1234)

        for attr in self.vanilla_bucket.current_attrs:
            if attr not in self.vanilla_bucket.retired_attrs:
                self.assertFalse(attr in self.vanilla_bucket.aggregated.keys())

    def test_prune(self):
        instances = [self._create_mock_tqdm(i) for i in range(10)]

        for i in instances:
            self.vanilla_bucket.update(i)

        for k in self.vanilla_bucket.known_instances.keys():
            self.vanilla_bucket.known_instances[k] = datetime.fromisoformat(
                "2000-01-01T00:00:00")

        self.vanilla_bucket.prune(1)
        self.assertEqual(
            len(self.vanilla_bucket.known_instances), 0)
        self.assertEqual(self.vanilla_bucket.retired_instances, len(instances))

    def test_export(self):
        instances = [self._create_mock_tqdm(i) for i in range(10)]

        for i in instances:
            self.vanilla_bucket.update(i)

        lines = list(self.vanilla_bucket.to_prometheus_lines())
        real_lines = [line for line in lines if not line.startswith("#")]

        # expect count to be
        # 1x active
        # 1x finished
        # 1x each indified attr
        all_attrs = set(self.vanilla_bucket.current_attrs +
                        self.vanilla_bucket.retired_attrs)
        self.assertEqual(len(real_lines), 2 + len(all_attrs))


if __name__ == '__main__':
    unittest.main()



from argparse import ArgumentParser

from release_tool.release import FullRelease, LocalRelease, DryRunRelease

safety_default = 'dry-run'
safety = {
    'full': FullRelease, # full shebang, commit, push, tag, etc
    'local': LocalRelease, # do commit, but dont push or have external effects
    safety_default: DryRunRelease # only touch local files
}

def get_parser():
    parser = ArgumentParser(description='Bump version')
    parser.add_argument('release_type', type=str, choices=DryRunRelease.version_components.keys(), help='Version component bump')
    parser.add_argument('method', type=str, choices=safety.keys(), default=safety_default, help=f"Release method (default: {safety_default})")
    
    parser.add_argument('--commit', action='store_true', help='Execute the updates')
    
    return parser


def main():
    parser = get_parser()
    
    opts = parser.parse_args()
    if opts.release_type not in DryRunRelease.version_components:
        parser.print_help()
        return

    release_class = safety[opts.method]
    release = release_class(opts.release_type)

    print(f"Starting release: {release}")
    release.do_release()


if __name__ == '__main__':
    main()
    


from multiprocessing import Queue
import traceback
from tqdm import tqdm 
from time import sleep

from tqdmpromproxy.internal.executor import TaskCountingPoolExecutor

def generator_init(lock, queue):
    tqdm.set_lock(lock)

    global tqdm_slot
    # if this fails the queue is empty and we should exit
    tqdm_slot = queue.get(timeout=1)

def create_threadpool(size:int=2, base: int = 0):
        offsets = Queue()

        for r in range(base, base+size):
            offsets.put(r)

        pool = TaskCountingPoolExecutor(size,
                                    initializer=generator_init,
                                    initargs=(tqdm.get_lock(), offsets))
    
        return pool


def queue_tasks(pool, quanity:int):
    names = ["Upload", "Download", "Gzip", "Bzip", "Tar", "Untar", "Copy", "Move", "Delete", "List"]
    try:
        for q in range(quanity):
           pool.submit(_off_thread_task, (names[q % len(names)]))
    except Exception as e:
        print(e)
        traceback.print_exc()
        pool.shutdown(wait=True)


def _off_thread_task(name:str, duration:int = 5, step:float = 0.2):
    
    with tqdm(total=duration, desc=name, position=tqdm_slot, leave=False) as pbar:
        for _ in range(int(duration / step)):
            sleep(step)
            pbar.update(step)


from concurrent.futures import ProcessPoolExecutor

class TaskCountingPoolExecutor(ProcessPoolExecutor):

    def __init__(self,  *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._completed_tasks = 0
        self._callback = None

    def set_callback(self, callback):
        self._callback = callback

    def submit(self, *args, **kwargs):
        future = super().submit(*args, **kwargs)
        future.add_done_callback(self._worker_is_done)
        return future

    def _worker_is_done(self, _):
        self._completed_tasks += 1

        if self._callback:
            self._callback()

    def get_completed_count(self):
        return self._completed_tasks



import logging
from multiprocessing import RLock, freeze_support
from time import sleep

from tqdm import tqdm as native_tqdm

from tqdmpromproxy import TqdmPrometheusProxy
from tqdmpromproxy.internal.creator import create_threadpool,queue_tasks

proxy = TqdmPrometheusProxy()

def main():
    # muliprocessing setup
    freeze_support()  # for Windows support
    native_tqdm.set_lock(RLock())  # for managing output contention
    
    proxy.start()
    proxy.add(native_tqdm())

    threads = 3
    depth = 10
    pool = create_threadpool(threads, base=3) # use slots 3, 4, 5
    queue_tasks(pool, threads * depth)

    try:
        for _ in native_tqdm(range(10), desc="Main Loop", position=0):
            for _ in native_tqdm(range(5), desc="Sub Loop", position=1, leave=False):
                for _ in native_tqdm(range(5), desc="Innnnner Loop", position=2, leave=False):
                    sleep(0.2)

    finally:
        proxy.stop()


if __name__ == "__main__":
    logging.basicConfig(filename="data/proxy.log",
                        filemode='w',
                        format='%(asctime)s,%(msecs)03d %(name)s %(levelname)s %(message)s',
                        datefmt='%Y-%m-%d %H:%M:%S',
                        level=logging.DEBUG
                        )
    main()



import os
import toml

from git import Repo

class DryRunRelease():
    version_components= {'major': 0, 'minor': 1, 'patch': 2}

    def __init__(self, bump_type):
        self.bump_type = bump_type

        self.from_version, self.to_version = DryRunRelease.determine_version_numbers(self.bump_type)

        os.makedirs('release', exist_ok=True)
    
    @staticmethod
    def determine_version_numbers(version_increment: str):
        toml_str= DryRunRelease.read_current_toml()

        version_str = toml_str['project']['version']
        version_parts = version_str.split('.')

        version_parts[DryRunRelease.version_components[version_increment]] = str(int(version_parts[DryRunRelease.version_components[version_increment]]) + 1)
        new_version = '.'.join(version_parts)

        toml_str['project']['version'] = new_version

        return (version_str, new_version)
    
    def __repr__(self):
        return f"{self.from_version} + {self.bump_type} => {self.to_version}"

    def do_release(self):
        self.bump_version()
        self.write_changelog()
        self.write_changes()
        self.create_tag()
        self.push()
        
    def bump_version(self):
        toml_content = self.read_current_toml()
        toml_content['project']['version'] = self.to_version

        with open('pyproject.toml', 'wt') as f:
            f.write(toml.dumps(toml_content))
        with open("release/version.new.txt", 'wt') as f:
            f.write(self.to_version)
        with open("release/version.old.txt", 'wt') as f:
            f.write(self.from_version)
        with open("release/changes.txt", 'wt') as f:
            f.write(self.get_changes())

    def write_changes(self):
        pass # only for reals

    def create_tag(self):
        pass# only for reals
    
    def push(self):
        pass# only for reals

    def get_changes(self, head=True):
        return DryRunRelease.get_release_changes(self.from_version, 'HEAD' if head else self.to_version)
    
    def write_changelog(self):
        existing = ""
        if os.path.exists('CHANGELOG.md'):
            with open('CHANGELOG.md', 'rt') as f:
                existing = f.read()
        
        with open('CHANGELOG.md', 'wt') as f:
            f.write(f"## {self.to_version}\n")
            f.write(self.get_changes())
            f.write("\n\n")
            f.write(existing)

    @staticmethod
    def read_current_toml():
        with open('pyproject.toml', 'rt') as f:
            toml_content = toml.loads(f.read())
        return toml_content

    @staticmethod
    def get_release_changes(from_version, to_version="HEAD"):
        r = Repo('.')

        prefix="release/"
        if to_version.upper().strip() == "HEAD":
            prefix=""

        o = r.git.log(f"release/{from_version}...{prefix}{to_version}", oneline=True)
        return o


class LocalRelease(DryRunRelease):

    def write_changes(self):
        super().write_changes()

        r = Repo('.')
        r.git.add('pyproject.toml')
        r.git.add('CHANGELOG.md')
        r.git.commit(message=f"Release [{self.bump_type}] version to {self.to_version}")

    def create_tag(self):
        r = Repo('.')
        r.git.tag(f"release/{self.to_version}")

class FullRelease(LocalRelease):
    def __init__(self, bump_type):
        super().__init__(bump_type)
    
    def bump_version(self):
        super().bump_version()

    def push(self):
        r = Repo('.')
        r.git.push('origin', '--tags')
        





from tqdmpromproxy.metric_server import AsyncMetricServer

from queue import Queue
import time
import logging


def main():
    q = Queue()
    s = AsyncMetricServer(q, 'localhost', 9000)

    s.start()

    try:

        for i in range(50):
            print('putting', i)
            q.put(f"test_metric {i}")
            time.sleep(.1)

    finally:
        s.stop()


if __name__ == '__main__':

    logging.basicConfig(filename="data/server.log",
                        filemode='w',
                        format='%(asctime)s,%(msecs)03d %(name)s %(levelname)s %(message)s',
                        datefmt='%Y-%m-%d %H:%M:%S',
                        level=logging.DEBUG
                        )

    main()
